@@ -1,5 +1,5 @@
 package AnyEvent::SlackRTM;
-$AnyEvent::SlackRTM::VERSION = '1.2';
+$AnyEvent::SlackRTM::VERSION = '1.3';
 use v5.14;
 
 # ABSTRACT: AnyEvent module for interacting with the Slack RTM API
@@ -11,7 +11,7 @@ use Furl;
 use JSON;
 use Try::Tiny;
 
-our $START_URL = 'https://slack.com/api/rtm.start';
+our $START_URL = 'https://slack.com/api/rtm.connect';
 
 
 sub new {
@@ -63,33 +63,41 @@ sub start {
     # Store this stuff in case we want it
     $self->{metadata} = $start;
 
-    my $wss    = $start->{url};
-    my $client = $self->{client};
+    # We've now asked to re-open the connection,
+    # so don't close again on timeout.
+    delete $self->{closed};
 
-    $client->connect($wss)->cb(sub {
+    $self->{client}->connect( $start->{url} )->cb( sub {
         my $client = shift;
 
-        my $conn = try {
-            $client->recv;
-        }
-        catch {
-            die $_;
-        };
-
+        delete $self->{finished};
         $self->{started}++;
         $self->{id} = 1;
 
-        $self->{conn} = $conn;
+        my $conn = $self->{conn} = $client->recv;
+        $conn->on( each_message => sub { $self->_handle_incoming(@_) } );
+        $conn->on( finish       => sub { $self->_handle_finish(@_) } );
 
-        $self->{pinger} = AnyEvent->timer(
-            after    => 60,
-            interval => 60,
-            cb       => sub { $self->ping },
+        my $started = localtime;
+        $self->{_last_keep_alive} = time;
+        $self->{keep_alive}       = AnyEvent->timer(
+            after    => 15,
+            interval => 15,
+            cb       => sub {
+                my $id    = $self->{id};
+                my $now   = time;
+                my $since = $now - $self->{_last_keep_alive};
+                if ( $since > 30 ) {
+                    # will trigger a finish, which will reconnect
+                    # if $self->{closed} is not set.
+                    $conn->close;
+                }
+                elsif ( $since > 10 ) {
+                    $self->ping( { keep_alive => $now } );
+                }
+            },
         );
-
-        $conn->on(each_message => sub { $self->_handle_incoming(@_) });
-        $conn->on(finish => sub { $self->_handle_finish(@_) });
-    });
+    } );
 }
 
 
@@ -165,6 +173,8 @@ sub _handle_incoming {
         croak "unable to decode incoming message: $message";
     };
 
+    $self->{_last_keep_alive} = time;
+
     # Handle errors when they occur
     if ($msg->{error}) {
         $self->_handle_error($conn, $msg);
@@ -222,16 +232,22 @@ sub _handle_other {
 sub _handle_finish {
     my ($self, $conn) = @_;
 
-    # Cancel the pinger
-    undef $self->{pinger};
+    # Cancel the keep_alive watchdog
+    undef $self->{keep_alive};
 
     $self->{finished}++;
 
     $self->_do('finish');
+
+    $self->start unless $self->{closed};
 }
 
 
-sub close { shift->{conn}->close }
+sub close {
+    my ($self) = @_;
+    $self->{closed}++;
+    $self->{conn}->close;
+}
 
 __END__
 
@@ -245,7 +261,7 @@ AnyEvent::SlackRTM - AnyEvent module for interacting with the Slack RTM API
 
 =head1 VERSION
 
-version 1.2
+version 1.3
 
 =head1 SYNOPSIS
 
@@ -330,11 +346,21 @@ This will establish the WebSocket connection to the Slack RTM service.
 
 You should have registered any events using L</on> before doing this or you may miss some events that arrive immediately.
 
+Sets up a "keep alive" timer,
+which triggers every 15 seconds to send a C<ping> message
+if there hasn't been any activity in the past 10 seconds.
+The C<ping> will trigger a C<pong> response,
+so there should be at least one message every 15 seconds.
+This will disconnect if no messages have been received in the past 30 seconds;
+however, it should trigger an automatic reconnect to keep the connection alive.
+
 =head2 metadata
 
     method metadata() returns HashRef
 
-The initial connection is established after calling the L<rtm.start|https://api.slack.com/methods/rtm.start> method on the web API. This returns some useful information, which is available here.
+The initial connection is established after calling the
+L<rtm.connect|https://api.slack.com/methods/rtm.connect> method on the web API.
+This returns some useful information, which is available here.
 
 This will only contain useful information I<after> L</start> is called.
 
@@ -404,7 +430,7 @@ Andrew Sterling Hanenkamp <hanenkamp@cpan.org>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2021 by Qubling Software LLC.
+This software is copyright (c) 2022 by Qubling Software LLC.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
